#! /usr/bin/env perl
#
# get_MSA_from_hhr.pl
# read a hhr file and generate an fas file of multiple sequence alignment containing the first k hits and another file in clustal format (the redundancy will be elimilated)
# Author: Jinchao YU
# Date: 2013-03-29
use strict;
use File::Spec;
use File::Path qw(mkpath);
use File::Basename;

my $usage = "Usage: get_MSA_from_hhr.pl -i <infile> [options]
-i : infile in format hhr
-k : choose the first k hits (5 by default)
-m : choose the multiple hits by a series of numbers, ex: -m 1,2,5..8,10
     In this case, option -k will be ignored
-d : output directory (the current directory by deafult)
-showk : show kbest-Viterbi indices k
\n";
my $dir_scripts = dirname(__FILE__)."/";
my $reformat = $dir_scripts."reformat.pl";
my $hhmakemodel_multiple = $dir_scripts."hhmakemodel_multiple.pl";
if(length(@ARGV) < 1) {die($usage);}
my $options = "";
for(my $i=0; $i<@ARGV; $i++) {$options.=" $ARGV[$i] ";}

my $infile = "";
my $k = 0;
my $outdir = "";
my $pickhits = "";
my $showk = 0; 

if($options=~m/(help)|(-h)/i) {die($usage);}
if($options=~s/-i\s+(\S+)/ /g) {
    $infile = $1; 
}
unless(length($infile)) {die($usage);}

if($options=~s/-showk/ /g) {
    $showk = 1; 
}

if($options=~s/-k\s+(\d+)/ /g) {
    $k = $1;
}
if($k<=1) {$k = 5;}

if($options=~s/-d\s+(\S+)/ /g) {
    $outdir = $1;
    unless(-d $outdir){mkpath($outdir);}   
}
unless(length($outdir)) {$outdir=File::Spec->curdir();}
if ($options=~s/-m\s+(\S+)/ /g) {$pickhits=$1;}


# note the k indices
open(INFILE,"<",$infile);
my %indices;
while(my $line=<INFILE>){
    if($line=~m/\s*(\d+)\s+k=(\d+)/){
        $indices{$1}=$2;
    }
    if($line=~m/^No 1/){
        last;}
}
close(INFILE);


sub FromTo{
    my $from;
    my $to;
    my $stringOfNumbers="";
    ($from, $to)=@_;
    if($from + 10000 < $to) {$stringOfNumbers="$from $to";}
    else {
	while($from<=$to){
        $stringOfNumbers .= "$from ";
	$from++;
        }
    }
    return $stringOfNumbers;
}

sub getNumber{
	my $number = $_[0];
	$number =~ s/_k.*//g;
	$number =~ s/\D//g;
	return $number;
}

my @output_indices;
if ($pickhits eq ""){
    for my $i(1..$k){
        if($showk == 1){
            push(@output_indices,"_".$i."_k=$indices{$i}");}
        else{
            if($indices{$i}==1) {push(@output_indices,"_(Vit)".$i);}
            else {push(@output_indices,"_".$i);}
            }
    }

}else{
    my $temp_pickhits = $pickhits;
    $temp_pickhits =~s/,/ /g;
    $temp_pickhits =~ s/(\d+)\.\.(\d+)/&FromTo($1,$2)/ge;
    my @pickhits = split(/\s+/,$temp_pickhits);

    foreach my $i(@pickhits){
        if($showk == 1){
            push(@output_indices,"_".$i."_k=$indices{$i}");}
        else{
            if($indices{$i}==1) {push(@output_indices,"_(Vit)".$i);}
            else {push(@output_indices,"_".$i);}
            }
    }
}
my @sorted_output_indices = sort { getNumber($a) <=> getNumber($b) } @output_indices;
my $filename = basename($infile);
$filename =~s/\.hhr//;
my $fasfile = File::Spec->catfile( $outdir, $filename ).".fasta";
my $clufile = File::Spec->catfile( $outdir, $filename ).".clu";
if ($pickhits eq ""){
    system("$hhmakemodel_multiple -i $infile -fas $fasfile -m 1..$k");
}
else{
    system("$hhmakemodel_multiple -i $infile -fas $fasfile -m $pickhits");
}

# modify the index in fasta file
my $fasfile_temp = $fasfile.".temp";
system("mv $fasfile $fasfile_temp");
open(INFILE,"<", $fasfile_temp) or die $!;
open(OUTFILE,">", $fasfile) or die $!;
my $j=0;
my $first = 1;
my $seqNameTemp;
my @seqName;
my %seq;
while(my $line=<INFILE>){
    if($line=~m/>\s*(\S{4,})/){
        if ($first==0){  #ignore the first sequence, which is the query's
        my $temp = $1;
        $line =~s/$temp/$temp$sorted_output_indices[$j]/;
        $j+=1;
        }
        else{ $first = 0;}
        $seqNameTemp = $line;
        push(@seqName,$line);
    }
    else{
    	$seq{$seqNameTemp} .= $line;
    }
}
my @printedSeq;
foreach my $sn(@seqName){
	unless(grep(/^$seq{$sn}$/, @printedSeq)){
		print OUTFILE $sn.$seq{$sn};
	}
	push(@printedSeq,$seq{$sn});
}
close(INFILE);
close(OUTFILE);
unlink($fasfile_temp);
system("$reformat $fasfile $clufile");
